{
    Project: Palette
    Company: Word in Action
             Copyright (C) 1995 by Jay Giganti.  All Rights Reserved.

    File    : Pals.Pas
    Author  : Jay Giganti
    Overview: To Modify the color palette and display a road to nowhere

                              Version Information
      Date    Ver  Comments
    ======== ===== ===========================================================
    00/00/95  1.00 Began work on this program
}

unit Pals;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls;

type
  TForm1 = class(TForm)
    Timer1: TTimer;
    Panel1: TPanel;
    Image1: TImage;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Timer1Timer(Sender: TObject);
  private
    { Private declarations }
    procedure CreatePal;
    procedure PaintImg(canv: Tcanvas);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

const
  Entries = 256;
  Clrs    = Trunc(Entries / 4);

var
  CurPal  : Array[0..Entries-1] of TPALETTEENTRY;
  hPal    : HPalette;
  hOldPal : HPalette;
  hImgDC  : THandle;
{==============================================================================
                                 CreatePal
Purpose:
Details:
==============================================================================}
procedure TForm1.CreatePal;
var
  nCntr : Integer;
  pPal  : PLOGPALETTE;
  lSize : LongInt;
begin

  lSize               := sizeof(TLogPalette) + Entries * sizeof(TPaletteEntry);
  pPal                := MemAlloc(lSize);
  pPal^.palVersion    := $300;
  pPal^.palNumEntries := Entries;

  {$R-}
  for nCntr := 0 to Clrs do
  begin
          { Set up the shades of Gray }
    pPal^.palPalEntry[nCntr].peRed   := nCntr + 128;
    pPal^.palPalEntry[nCntr].peGreen := nCntr + 128;
    pPal^.palPalEntry[nCntr].peBlue  := nCntr + 128;
    pPal^.palPalEntry[nCntr].peFlags := PC_RESERVED;

          { Set up the shades of Red }
    pPal^.palPalEntry[nCntr + Clrs].peRed   := nCntr + 128;
    pPal^.palPalEntry[nCntr + Clrs].peGreen := 0;
    pPal^.palPalEntry[nCntr + Clrs].peBlue  := 0;
    pPal^.palPalEntry[nCntr + Clrs].peFlags := PC_RESERVED;

           { Set up the shades of Green }
    pPal^.palPalEntry[nCntr + Clrs*2].peRed   := 0;
    pPal^.palPalEntry[nCntr + Clrs*2].peGreen := nCntr + 128;
    pPal^.palPalEntry[nCntr + Clrs*2].peBlue  := 0;
    pPal^.palPalEntry[nCntr + Clrs*2].peFlags := PC_RESERVED;

           { Set up the shades of Blue }
    pPal^.palPalEntry[nCntr + Clrs*3].peRed   := 0;
    pPal^.palPalEntry[nCntr + Clrs*3].peGreen := 0;
    pPal^.palPalEntry[nCntr + Clrs*3].peBlue  := nCntr + 128;
    pPal^.palPalEntry[nCntr + Clrs*3].peFlags := PC_RESERVED;
   end;

   for nCntr := 0 to Entries - 1 do
    CurPal[nCntr] := pPal^.palPalEntry[nCntr];

   {$R+}
   hPal := CreatePalette(pPal^);
   FreeMem(pPal, lSize);
end;
{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
var
 wReal : WORD;
 bmp   : TBitmap;
begin
  CreatePal;
  Width      := 640;
  Height     := 480;
  bmp        := TBitmap.Create;
  bmp.Width  := ClientWidth;
  bmp.Height := ClientHeight;
  hImgDC     := Image1.Canvas.Handle;
  hOldPal    := SelectPalette(hImgDC, hPal, FALSE);
  wReal      := RealizePalette(hImgDC);
  Panel1.Caption:= Format('Realized %u Entries out of %u',
                          [wReal, Entries]);
  PaintImg(Image1.Canvas);
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  SelectPalette(hImgDC, hOldPal, FALSE);
  DeleteObject(hPal);
end;

procedure TForm1.Timer1Timer(Sender: TObject);
var
  Pal   : TPALETTEENTRY;
  nCntr : Integer;
begin
  Pal := CurPal[0];
  for nCntr := 0 to Entries - 2 do
    CurPal[nCntr] := CurPal[nCntr + 1];
  CurPal[nCntr] := Pal;

  AnimatePalette(hPal, 0, Entries, CurPal);
end;

procedure TForm1.PaintImg(canv: Tcanvas);
var
 Clr   : LongInt;
 y,
 x1,
 x2    : Integer;
begin
  x1  := ClientWidth DIV 2 - 10;
  x2  := x1;
  Clr := 1;

  for y := 60 to ClientHeight - 60 do
  begin
    Canv.Pen.Color := PaletteIndex(Clr);
    Canv.MoveTo(x1, y);
    Canv.LineTo(x2, y);

    Dec(x1);
    if x1 < 0 then
      x1 := 0;

    Inc(x2);
    if (x2 > ClientWidth) then
      x2 := ClientWidth;

    if ((y mod 5) = 0) then
      Inc(Clr);

    if (clr > Entries) then
      Clr := 1;
  end;
end;
end.
